﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;

namespace Common.MVVM.Binding
{
    public abstract class BindableBase : DependencyObject, INotifyPropertyChanged
    {
        private bool _isDirty;

        /// <summary>
        /// 
        /// </summary>
        public bool IsDirty
        {
            get { return _isDirty; }
            set { SetFieldDoesNotCheckIsDirty(ref _isDirty, value, nameof(IsDirty)); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }


        protected bool SetFieldDoesNotCheckIsDirty<T>(ref T field, T value, string propertyName)
        {
            if (EqualityComparer<T>.Default.Equals(field, value)) return false;
            field = value;
            OnPropertyChanged(propertyName);
            return true;
        }


        protected bool SetField<T>(ref T field, T value, string propertyName)
        {
            if (EqualityComparer<T>.Default.Equals(field, value)) return false;
            field = value;
            OnPropertyChanged(propertyName);
            IsDirty = PropertyHasChangedCheckModelIsDirty();
            return true;
        }

        /// <summary>
        /// Checks if this model is dirty i.e. has been changed compared to the old model object.
        /// </summary>
        /// <remarks>Implemented in the subclass.</remarks>
        /// <returns>True if model is dirty, otherwise false.</returns>
        protected abstract bool PropertyHasChangedCheckModelIsDirty();
        
    }
}
