﻿using System;
using System.ComponentModel;
using System.Threading.Tasks;
using Common.MVVM.Binding;

namespace Common.MVVM.Models
{
    /// <summary>
    /// 
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public abstract class BusinessBase<T> : BindableBase where T : BusinessBase<T>
    {
        public bool IsSaving { get; set; }

        
        /// <summary>
        /// Runs a task asynchronously when the calling method return type cannot be Task (e.g. in a constructor).
        /// </summary>
        /// <param name="task">A task to run asynchronously.</param>
        protected void RunAsyncTask(Task task) => System.Windows.Application.Current.Dispatcher.Invoke(async () => await task);



        public event EventHandler<PropertyChangedEventArgs> ModelChanged;

        protected virtual void ModelHasChanged(string propertyName)
        {
            var propertyChangedEventArgs = new PropertyChangedEventArgs(propertyName);

            ModelChanged?.Invoke(this, propertyChangedEventArgs);
        }


        /// <summary>
        /// Sets the object to be clean i.e. not dirty.
        /// </summary>
        protected void SetObjectClean()
        {
            IsDirty = false;
        }

        /// <summary>
        /// Sets the object to be dirty i.e. object has changed.
        /// </summary>
        protected void SetObjectDirty()
        {
            IsDirty = true;
        }


    }
}
