﻿using System;
using System.Reflection;
using Infragistics.Controls.Interactions;

namespace Common.MVVM.ViewModels
{
    public class ViewModelWithDialogWindowBase<T> : ViewModelBase<T>, IViewModelWithDialogWindowBase where T : class
    {
        private XamDialogWindow _xamDialogWindow;

        /// <summary>
        /// Holds an instance of the Infragistics XamDialogWindow.
        /// </summary>
        public XamDialogWindow XamDialogWindow
        {
            get { return _xamDialogWindow; }
            private set => _xamDialogWindow = value;
        }

        public void CloseXamDialogWindow()
        {
            _xamDialogWindow?.Close();
            _xamDialogWindow = null;
        }






        public void ShowXamDialogWindow()
        {
            _xamDialogWindow = CreateXamDialogWindowInstance();

            if (_xamDialogWindow == null) return;

            _xamDialogWindow.IsModal = false;
            _xamDialogWindow.StartupPosition = StartupPosition.Center;
            _xamDialogWindow.Left = 1000;
            _xamDialogWindow.Top = 500;
            _xamDialogWindow.IsResizable = true;
            _xamDialogWindow.IsMoveable = true;
            _xamDialogWindow.WindowState = WindowState.Normal;


            MethodInfo method = _xamDialogWindow.GetType().GetMethod("ShowDialog");

            if (method == null) return;

            method.Invoke(_xamDialogWindow, null);
        }



        private XamDialogWindow CreateXamDialogWindowInstance()
        {
            string viewModelName = this.ToString();

            if (viewModelName.Contains("Model"))
            {
                string viewName = viewModelName.Replace("Model", string.Empty, false,
                    System.Globalization.CultureInfo.InvariantCulture);

                Assembly assembly = Assembly.GetAssembly(this.GetType());

                XamDialogWindow xamDialogWindowInstance = CreateInstance(assembly.FullName, viewName);

                if (xamDialogWindowInstance == null)
                {
                    throw new InvalidOperationException("xamDialogWindowInstance is null");
                }

                Type myDialogWindowType = xamDialogWindowInstance.GetType().BaseType;

                PropertyInfo propertyInfo = myDialogWindowType.GetProperty("DataContext");

                // Sets the view's DataContext property to the viewmodel.
                propertyInfo.SetValue(xamDialogWindowInstance, this);

                return xamDialogWindowInstance;
            }

            throw new InvalidOperationException("viewModelName does not contain the word Model");
        }



        /// <summary>
        /// Creates an instance of type T using the assembly name and type name.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="assemblyName"></param>
        /// <param name="typeName"></param>
        /// <remarks>Type T can be an interface.</remarks>
        /// <returns>An instance of type T otherwise the default(T).</returns>
        private XamDialogWindow CreateInstance(string assemblyName, string typeName)
        {
            var objectHandle = Activator.CreateInstance(assemblyName, typeName);

            dynamic value = objectHandle.Unwrap();

            return value;
        }

        //public override void Initialize()
        //{
        //    base.Initialize();

        //}


    }
}
